import axios, { InternalAxiosRequestConfig, AxiosResponse, AxiosError } from 'axios'
// import { useUserStoreHook } from '@/store/modules/user'
import { LOGIN_URL, SMS_URL } from '@/apis/url'
import { message as Message } from 'ant-design-vue'
import clientBasic from '@/common/client'
import logout from '@/common/logout'
// 定义返回数据格式
interface ApiResponse<T = any> {
  code: number
  msg: string
  data: T // 这里定义请求返回data数据类型
}
// 定义axios函数接口
interface ServiceInterface {
  <M = any, T extends object = any>(url: string, data?: T): Promise<M>
}
// 创建 axios 实例
const service = axios.create({
  baseURL: import.meta.env.VITE_APP_AXIOS_BASE_URL,
  timeout: 50000
  // headers: { 'Content-Type': 'application/json;charset=utf-8' }
})
// 请求拦截器
service.interceptors.request.use(
  (config: InternalAxiosRequestConfig) => {
    const { url } = config
    const uri = url!.split('?')[0]
    // 如果是登录和刷新token，添加专用token
    // 否则使用正常的 token
    if ([LOGIN_URL.LOGIN].includes(uri)) {
      config.headers.Authorization = `Basic ${clientBasic}`
    } else if (
      ![
        LOGIN_URL.REGISTER,
        LOGIN_URL.RESET,
        SMS_URL.IMAGE_CODE,
        SMS_URL.SMS_REGISTER,
        SMS_URL.SMS_RESET
      ].includes(uri)
    ) {
      config.headers.Authorization = `Bearer ${localStorage.getItem('access_token')}`
    }
    return config
  },
  (error: AxiosError) => {
    return Promise.reject(error)
  }
)
// 响应拦截器
service.interceptors.response.use(
  (response: AxiosResponse) => {
    if (response.request.responseType === 'blob') {
      return response.data
    }
    const responseData: ApiResponse = response.data
    const { code, msg, data } = responseData
    if (code === 2000) {
      return Promise.resolve(data)
    }
    if (code === 4001) {
      Message.error(msg || '系统出错')
      return Promise.reject(new Error(msg || 'Error'))
    }
    return Promise.reject(response.data)
  },
  (err: AxiosError) => {
    let message = ''
    const states = err.response!.status
    switch (states) {
      case 400:
        message = '请求错误(400)'
        break
      case 401:
        message = '登录过期，请重新登录！'
        logout()
        break
      case 403:
        message = '权限不足'
        break
      case 404:
        message = '请求出错(404)'
        break
      case 408:
        message = '请求超时(408)'
        break
      case 500:
        message = '服务器错误(500)'
        break
      case 501:
        message = '服务未实现(501)'
        break
      case 502:
        message = '网络错误(502)'
        break
      case 503:
        message = '服务不可用(503)'
        break
      case 504:
        message = '网络超时(504)'
        break
      case 505:
        message = 'HTTP版本不受支持(505)'
        break
      default:
        message = `连接出错(${err.response!.status})!`
    }
    Message.error(message)
    console.log('message :>> ', message)
    return Promise.reject({ msg: message, states })
  }
)
// 导出 axios 实例
const getFn: ServiceInterface = (url, data) => {
  return service.get(url, { params: data })
}
const postFn: ServiceInterface = async (url, data) => {
  return await service.post(url, data)
}
const deleteFn: ServiceInterface = async (url, data) => {
  return await service.delete(url, data)
}
const putFn: ServiceInterface = async (url, data) => {
  return await service.put(url, data)
}
const postDownload: ServiceInterface = async (url, data) => {
  return await service.post(url, data, { responseType: 'blob' })
}
const getDownload: ServiceInterface = async (url, params) => {
  return await service.get(url, { params, responseType: 'blob' })
}
export default {
  // async get<T extends AxiosRequestConfig<any>, M>(url: string, params?: T): AxiosPromise<M> {
  //   return await service.get(url, { params })
  // },
  // async post<M, T = any>(url: string, data?: T): AxiosPromise<M> {
  //   return await service.post(url, data)
  // },
  // async put<T, M>(url: string, data: T): AxiosPromise<M> {
  //   return await service.put(url, data)
  // },
  // async postDownload(url, data) {
  //   return await service.post(url, data, { responseType: 'blob' })
  // },
  // async getDownload(url, params) {
  //   return await service.get(url, { params, responseType: 'blob' })
  // },
  get: getFn,
  post: postFn,
  delete: deleteFn,
  put: putFn,
  postDownload,
  getDownload
}
