import xHttp from '@/apis/Xhttp'
import { searchResult, searchData } from './types'
import { FORM_DATA_URL } from '@/apis/url'
import { saveAs } from 'file-saver'
import { message } from 'ant-design-vue'

export default {
  /**
   * 表单数据查询接口
   * @param {*} formName 表名
   * @param {*} param 查询参数
   * @returns
   */
  async searchFormData(formName: string, param?: searchData) {
    return await xHttp.post<searchResult, searchData>(FORM_DATA_URL.SEARCH(formName), param)
  },
  /**
   * 表单数据添加接口
   * @param {*} formName 表名
   * @param {*} formData 表单数据
   * @returns
   */

  async addFormData(formName, formData) {
    return await xHttp.post(FORM_DATA_URL.ADD(formName), formData)
  },
  /**
   * 表单数据批量添加接口
   * @param {*} formName 表名
   * @param {*} formData 表单数据
   * @returns
   */

  async batchAddFormData(formName, formDataList) {
    return await xHttp.post(FORM_DATA_URL.BATCH(formName), formDataList)
  },

  /**
   * 关联表单数据查询
   * @param {*} formName 表名
   * @param {*} param 查询参数
   * @returns
   */

  async joinSearchFormData(formName, param) {
    return await xHttp.post(FORM_DATA_URL.JOIN_SEARCH(formName), param)
  },
  /**
   * 表单数据查询单条
   * @param {*} formName 表名
   * @param {*} id 单条数据id
   * @param {*} resolveObj 是否翻译字典和显示子表
   * @returns
   */

  async searchFormDataById(formName, id, resolveObj) {
    const resolveDic = resolveObj?.resolveDic || false
    const resolveSubItems = resolveObj?.resolveSubItems || false
    return await xHttp.get(FORM_DATA_URL.SEARCH_BY_ID(formName, id, resolveDic, resolveSubItems))
  },

  /**
   * 表单数据删除
   * @param {*} formName 表名
   * @param {*} id 单条数据id
   * @returns
   */

  async deleteFormDataById(formName: string, id: string) {
    return await xHttp.delete(FORM_DATA_URL.DELETE_BY_ID(formName, id))
  },
  /**
   * 表单数据批量删除
   * @param {*} formName 表名
   * @param {*} ids 数据id数组
   * @returns
   */
  async bathDeleteFormData(formName: string, ids: string[]) {
    return await xHttp.delete(FORM_DATA_URL.BATCH(formName), {
      data: { dataIds: ids }
    })
  },
  /**
   * 表单数据更新接口
   * @param {*} formName 表名
   * @param {*} id 单条数据id
   * @returns
   */
  async updateFormData(formName, formData) {
    return await xHttp.put(FORM_DATA_URL.UPDATE(formName), formData)
  },
  /**
   * 导入表单数据
   * @param {*} formName 表名
   * @param {*} file 导入文件
   * @returns
   */
  async importFormData(formName: string, file) {
    const formData = new FormData()
    formData.append('file', file)
    return await xHttp.post(FORM_DATA_URL.IMPORT(formName), formData)
  },
  /**
   * 表单数据更新状态接口
   * @param {*} formName 表名
   * @param {*} dataId 单条数据id
   * @param {*} formData 参数
   * @returns
   */
  async updateState(formName, dataId, formData) {
    return await xHttp.put(FORM_DATA_URL.UPDATE_STATE(formName, dataId), formData)
  },
  /**
   * 表单数据批量修改状态
   * @param {*} formName 表名
   * @returns
   */
  async batchUpdateState(formName, data) {
    return await xHttp.post(FORM_DATA_URL.BATCH_STATE(formName), data)
  },
  /**
   * 表单数据根据过滤条件批量修改状态
   * @param {*} formName 表名
   * @returns
   */
  async filterBatchUpdateState(formName, data) {
    return await xHttp.post(FORM_DATA_URL.BATCH_STATE_FILTER(formName), data)
  },
  /**
   * 表单数据操作日志接口
   * @param {*} formName 表名
   * @param {*} dataId 单条数据id
   * @returns
   */
  async searchOperationLogs(formName, dataId, param) {
    return await xHttp.get(FORM_DATA_URL.SEARCH_STATE(formName, dataId, param))
  },
  /**
   * 表单数据统计
   * @param {*} formName 表名
   * @param {*} data
   * {
    "aggregates": [["农场类型", "count"]],
    "groupFields":["pcode"],
    "filter":["=", "农场类型", "种植"]
    }
   * @returns
   */
  async formDataCount(formName, data) {
    return await xHttp.post(FORM_DATA_URL.COUNT(formName), data)
  },
  /**
   * 表单数据下载
   * @param {string} name 文件名
   * @param {string} formName 表名称
   * @param {object} data 过滤条件
   */
  async dataDownload(name, formName, data = {}) {
    message.info(`开始下载：${name}`)
    return xHttp.postDownload(`form/${formName}/download`, data).then((i) => {
      saveAs(i, name)
    })
  },
  /**
   * 导入数据（附带额外信息）
   * @param {string} formName 表名称
   * @param {file} file 上传Excel表格   表头为label   重复label使用spareLabel
   * @param {object} data 自带数据    key值为name
   * @param {array} fieldAssemble  数组 需要拼接参数  多个sourcefields 以delimiter为连接符组装，然后赋值给targetField
   *  示例 {"sourceFields":["field1","field2"],"delimiter":"-","targetField":"field3"}
   */
  async formexImport(formName, file, data, fieldAssemble) {
    const formData = new FormData()
    formData.append('file', file)
    if (data) {
      formData.append('data', JSON.stringify(data))
    }
    if (fieldAssemble) {
      formData.append('fieldAssemble', JSON.stringify(fieldAssemble))
    }
    return xHttp.post(`/formex/${formName}/import`, formData)
  },
  /**
   * 表单word下载
   * @param {string} formName 表名称
   * @param {object} data 表单数据 {"dataId": "数据id", "subForms": [{ "formName": "物料使用情况表"}]}
   * @param {string} name  下载word名称
   */
  async downloadWord(formName, data, name) {
    if (!name) name = `${formName}.docx`
    message.info(`开始下载：${name}`)
    return xHttp.getDownload(`/extend/word/${formName}/download`, data).then((i) => {
      saveAs(i, name)
    })
  },
  /**
   * 表单pdf查看
   * @param {string} formName 表名称
   * @param {object} data 表单数据 {"dataId": "数据id", "subForms": [{ "formName": "物料使用情况表"}]}
   */
  viewPdfUrl(formName, data) {
    const { dataId, subForms } = data
    let params = `dataId=${dataId}`
    if (subForms) {
      params += `&subForms=${encodeURIComponent(JSON.stringify(subForms))}`
    }
    return `${import.meta.env.VITE_APP_AXIOS_BASE_URL}/extend/pdf/${formName}/download?${params}`
  }
}
